import sys

class Exception:
    def __init__(self, name, reason):
        self.name = name
        self.reason = reason
    
    def execute(self):
        print(f"BooSQL={self.name}\n {self.reason}")
        sys.exit(0)


from .initializer import Initializer
import sys

class Database:
    def __init__(self, path:str="my_database.db", obfuscate:bool=True):
        self.initer = Initializer("\\".join(sys.argv[0].split("\\")[:-1])+f"\\{path}", obfuscate)
        self.manager = self.initer._manager
    
    def create(self):
        self.initer.create_file()
    
    def open_prepared_workspace(self):
        self.manager.open_workspace()
    
    def create_workspace(self, name):
        self.manager.create_workspace(name)
    
    def _get_items(self):
        return self.manager._items
    
    def get_workspace_name(self):
        return self.manager.__wname
    
    def add_item(self, item:list=[]):
        self.manager.add_item(item)
    
    def get_item(self, index:int=-1):
        if not index == -1:
            return self.manager.get_item(index)
    
    def edit_item(self, index:int=-1, new_item:list=[]):
        if not index == -1:
            return self.manager.replace_item(new_item, index)
    
    def remove_item(self, index:int=-1):
        if not index == -1:
            self.manager.remove_item(index)
    
    def allow_edit(self):
        self.initer.allow_manage()
    
    def disallow_edit(self):
        self.initer.disallow_manage()


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['boosql', 'boosql.exceptions']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'boosql',
    'version': '0.9.9',
    'description': '',
    'long_description': None,
    'author': 'yer',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
}


setup(**setup_kwargs)



from .exceptions.invalidpath import InvalidPathException
from .exceptions.unknownerrorfile import UnknownErrorFileException
from .manager import Manager
from base64 import *
from binascii import *

class Initializer:
    def __init__(self, path, obfuscate):
        self.path = path
        self.__obf = obfuscate
        
        self.invalid_path = InvalidPathException(self.path)
        self.uef = UnknownErrorFileException(self.path)
        self._manager = Manager(self)
    
    def create_file(self):
        try:
            with open(self.path, 'w', encoding='cp037') as db:
                db.write(b85encode("BOOSQL-FILE-DB-TYPE.INIT\nSTART".encode()).hex() if self.__obf else "BOOSQL-FILE-DB-TYPE.INIT\nSTART")
                db.close()
            self._manager.read_file()
        except:
            self.uef.display()

    def open_file(self):
        try:
            with open(self.path, 'r', encoding='cp037') as db:
                result = b85decode(bytes.fromhex(db.read()).decode()).decode() if self.__obf else db.read()
                db.close()
            return result
        except:
            self.invalid_path.display()
    
    def write(self, string):
        try:
            with open(self.path, 'w', encoding='cp037') as db:
                db.write(b85encode(string.encode()).hex() if self.__obf else string)
                db.close()
        except:
            self.uef.display()
    
    def allow_manage(self):
        self._manager.allowed = True
        self._manager.path = self.path
    
    def disallow_manage(self):
        self._manager.allowed = False
        self._manager.path = ""